@@ -43,7 +43,9 @@ exec("from "+prefix+"starfishXfn import getBalanceSheet")
 #from starfishX.starfishXfn import getBalanceSheetCompare
 exec("from "+prefix+"starfishXfn import getBalanceSheetCompare")
 
-exec("from "+prefix+"starfishXfn import getProfitAbility")  #ยกเลิกก่อน  เพราะ wsj หยุดให้บริการแบบ free
+#exec("from "+prefix+"starfishXfn import getProfitAbility")  #ยกเลิกก่อน  เพราะ wsj หยุดให้บริการแบบ free
+exec("from "+prefix+"getProfitAbility import getProfitAbility")  #ยกเลิกก่อน  เพราะ wsj หยุดให้บริการแบบ free
+
 #from starfishX.starfishXfn import getMemberOfIndex
 exec("from "+prefix+"starfishXfn import getMemberOfIndex")
 
@@ -82,6 +84,9 @@ exec("from "+prefix+"depreciationAndAmortisation import DA")
 #from starfishX.getEVEBITDA import getEVEBITDA
 exec("from "+prefix+"getEVEBITDA import getEVEBITDA")
 
+#from starfishX.getEVEBITDA import getEVEBITDA
+exec("from "+prefix+"getEVEBITDA import getEVEBIT")
+
 #from starfishX.getEVEBITDA import analyseEVEBITDA
 exec("from "+prefix+"getEVEBITDA import analyseEVEBITDA")
 
@@ -129,6 +134,7 @@ exec("from "+prefix+"AD import viewAnomalyDetectionReportAllType")
 
 #from starfishX.Ke import governmentBond10year
 exec("from "+prefix+"Ke import thaibma")
+exec("from "+prefix+"Ke import thaibma_series")
 exec("from "+prefix+"Ke import BondType") #enum bondType
 
 exec("from "+prefix+"EYG import EarningYieldGap")
@@ -158,6 +164,12 @@ exec("from "+prefix+"shortsale import shortsaleByDay")
 #from starfishX.starfishXfn import loadHistData
 exec("from "+prefix+"starfishXfn import fillHistData")
 
-__version__ = 0.155499
+exec("from "+prefix+"utility import *")
+
+exec("from "+prefix+"risk_kit import *")
+
+
+
+__version__ = 0.155504
 
 __doc__ = """starfishX Version จัดเตรียมข้อมูลสำหรับประมวลผลการลงทุนในตลาดหุ้นไทย """
\ No newline at end of file
