@@ -27,7 +27,8 @@ from tqdm import tqdm
 #สำหรับ Monthly Return #######
 import seaborn as sns
 import datetime
-from datetime import datetime as dtt
+
+from datetime import datetime as dtt #ใช้ในฟังก์ชั่น fillHistData 
 
 import urllib3
 urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
@@ -66,7 +67,7 @@ exec("from "+prefix+"listSecurities import listSecurities")
 
 
 ########### load ราคาย้อนหลัง ######################
-def loadHistData(symbol,start,end="",Volume=False,OHLC=False):
+def loadHistData(symbol,start,end="",Volume=False,OHLC=False,US=False,col_clean=True):
     """
     Returns DataFrame ราคาปิดและปริมาณการซื้อขาย 
     
@@ -80,46 +81,162 @@ def loadHistData(symbol,start,end="",Volume=False,OHLC=False):
     
     arg5 (Boolean): แสดง OHLC หรือไม่ ถ้าไม่ระบุจะเป็น Close Price เท่านั้น
 
+    arg6 (Boolean): หุ้น USA หรือหรือไทย หากหุ้นไทยไม่ต้องระบุ
+
+    arg7 (Boolean): ต้องการ clean หัวของคอลัมน์หรือไม่ เช่น ':Close' เป็นต้น
     """ 
     if(end==""):
      now = datetime.datetime.now()
      end = (str(now.year).strip()+"-"+str(now.month).strip()+"-"+str(now.day).strip())
     
     endDate = end
-    if(isinstance(symbol, list)):
-      print(symbol)
-      qs = crateQueryString(symbol,Volume=Volume,OHLC=OHLC)
-      k = qs.replace(".bk","")
-      symbol = k.split(",")
-      #if(Volume==True):
-      #  tmpsymbol = []
-      #  for i in symbol:
-      #    tmpsymbol.append(i)
-      #    tmpsymbol.append(i+":Volume")
-      #  symbol = tmpsymbol
 
 
+    ##### ตรวจว่าเป็นดัชนีไหม 18 มิถุนายน 2020
+    contain_index = False  
+    symbol_index_only = []
+    symbol_stock_only = []
+    ##### 
+    if(isinstance(symbol, list)):
+      
+      ############## block และคัดกรองส่วนของ index เท่านั้นก่อน ##########
+      for i in range(len(symbol)):
+        if(symbol[i][0]=="^"):
+            symbol_index_only.append(symbol[i].replace("^",""))
+            contain_index = True
+        else:
+            symbol_stock_only.append(symbol[i].replace("^",""))
+      ############## endblock และคัดกรองส่วนของ index เท่านั้นก่อน ##########
+
+     
+      
+      #return 0
+      #print(symbol)
+      qs = 0
+      if(len(symbol_stock_only)>0):
+       qs = crateQueryString(symbol_stock_only,Volume=Volume,OHLC=OHLC)
+       if(US==False):
+        k = qs.replace(".bk","")
+       symbol_stock_only = k.split(",")
+     
+       #if(Volume==True):
+       #  tmpsymbol = []
+       #  for i in symbol:
+       #    tmpsymbol.append(i)
+       #    tmpsymbol.append(i+":Volume")
+       #  symbol = tmpsymbol
+
+    ########## จบส่วนเช็คแบบ List #########
+
     if(isinstance(symbol, str)):
-      qs = symbol+".bk:Close"
+      ##### ตรวจว่าเป็นดัชนีไหม 18 มิถุนายน 2020 
+      if(symbol[0]=="^"):
+        symbol = symbol.replace("^","")
+        df = loadHistDataIndex(symbol,start)
+        return df #ถ้าเป็น str แล้วเป็นชนิด index คืนค่าออกไปเลย -- 18 มิถุนายน 2020
+        ###################################
+        ###################################
+
+      if(US==False):
+        qs = symbol+".bk:Close"
+      else:
+        qs = symbol  
       tmpsymbol = [symbol]
       if(OHLC==True):
-        qs = symbol+".bk:Open"+","+symbol+".bk:High"+","+symbol+".bk:Low"+","+symbol+".bk:Close" 
+        if(US==False):
+          qs = symbol+".bk:Open"+","+symbol+".bk:High"+","+symbol+".bk:Low"+","+symbol+".bk:Close" 
+        else:
+          qs = symbol+":Open"+","+symbol+":High"+","+symbol+":Low"+","+symbol+":Close"  
+
         tmpsymbol = ["Open","High","Low","Close"]  
       if(Volume==True):
-         qs+=","+symbol+".bk:Volume"
+         if(US==False):
+          qs+=","+symbol+".bk:Volume"
+         else:
+          qs+=","+symbol
+
          #symbol = [symbol,symbol+":Volume"]
          tmpsymbol.append("Volume")
       else:   
          symbol = [symbol]
       
       symbol = tmpsymbol
-    
+    #จบส่วนของเช็ค str
+
  
-    df = loaddata.get(qs,start=start,end=endDate)
-    df.columns = symbol
-    return df
+    if(qs!=0): #คือมีชุด query String ของหุ้นรายตัว*  ต่อมาสำเร็จ กรณีไม่สำเร็จเช่น ["^set","^set50"]
+     try:
+      df = loaddata.get(qs,start=start,end=endDate)
+      df.columns = symbol_stock_only #symbol
+      if(col_clean):
+            df = col_cleanFn(df)
+     except:
+       print("Can't Load Data")
+       return 0   
+ 
+    if(contain_index):
+       rpIndex = loadHistDataIndex(symbol_index_only,start=start)
+       if(qs!=0):
+         tmp = pd.merge(df,rpIndex,on='Date',how='inner')
+         if(col_clean):
+            tmp = col_cleanFn(tmp)
+         return tmp   
+       else:
+         return rpIndex  
+
+
+    ##### จบ กิจกรรม #####
+    return df  
+    
 ##################################################
 
+#### col clean ####
+def col_cleanFn(df):
+ tmp_col = []
+ for i in df.columns:
+  tmp_col.append(i.replace(":Close","").upper())   
+ df.columns = tmp_col
+ return df
+##################
+
+############## update 18 มีถุนายน 2020 ############# 
+############## รุ่นแก้ โหลดข้อมูลดัชนี #############   
+from datetime import datetime as dt
+def loadHistDataIndex(symbol,start):
+ if(type(symbol)==str):
+   symbol = symbol.upper()
+ if(type(symbol)==list):
+   symbol = [i.upper() for i in symbol]   
+
+ #url
+ url = "https://www.set.or.th/static/mktstat/setindex.csv"   
+ headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}   
+
+ ##gen filename
+ date = dt.now().strftime('%Y-%m-%d')
+ filename = "setindex_"+date+".csv"
+ r = requests.get(url,headers=headers)
+
+ # open method to open a file on your system and write the contents
+ with open(filename, "wb") as code:
+    code.write(r.content)
+ ##############################################
+ df = pd.read_csv(filename,date_parser=["Date"] ,parse_dates=True,skipinitialspace=True)
+ 
+ ##############################################   
+ #specify input format '%d-%m-%Y' and output format '%Y-%m-%d' 
+ #or change output as desired i.e. %d/%m/%Y to give dd/mm/yyyy
+ #https://stackoverflow.com/questions/52089360/how-to-change-dd-mm-yyyy-date-format-to-yyyy-dd-mm-in-pandas/52114869#52114869
+ df['Date'] = pd.to_datetime(df['Date'],format='%d/%m/%Y').dt.strftime('%Y-%m-%d')   
+ df["Date"] = pd.to_datetime(df['Date'])
+ df = df.set_index("Date")
+    
+ if(type(symbol)==str):
+   return df[[symbol]][df.index>=start]
+ if(type(symbol)==list):
+   return df[symbol][df.index>=start]
+
+############## end รุ่นแก้ โหลดข้อมูลดัชนี #############   
 
 ############  ผู้ถือหุ้น Shareholders #################
 def listShareholders_Preprocess(symbol,eventdate):
@@ -128,7 +245,7 @@ def listShareholders_Preprocess(symbol,eventdate):
   symbolMofi = symbol.replace("&","%26")
     
   url = "https://www.set.or.th/set/companyholder.do?symbol="+symbolMofi
-  r = requests.get(url,headers)
+  r = requests.post(url,headers)
   soup = BeautifulSoup(r.content, "lxml")  
 
   #k = soup.find_all("table",{"class":"table table-hover table-info-wrap"})
@@ -145,13 +262,13 @@ def listShareholders_Preprocess(symbol,eventdate):
   for i in k[0].find_all("tr"):
    tr = i.find_all("td")
    name    = (tr[1].text.strip())
-   Shares  = (tr[2].text.strip()) 
+   Shares  = (tr[2].text.strip().replace(",","")) 
    Percent = (tr[3].text.strip())
     
    dsSymbol.append(symbol)   
    dsName.append(name)
-   dsShares.append(Shares)
-   dsPercent.append(Percent) 
+   dsShares.append(float(Shares))
+   dsPercent.append(float(Percent))
     
   
   df = pd.DataFrame({"Symbol":dsSymbol,"Name":dsName,"Share":dsShares,"Percent":dsPercent}) 
@@ -172,7 +289,7 @@ def listShareholders_Preprocess(symbol,eventdate):
     df["Date"] = pd.to_datetime(date)
     df["Type"] = k_type
 
-    
+  pd.options.display.float_format = '{:,.2f}'.format  
   return df
 
 def listShareholders(symbol,csv="",eventdate=False): 
@@ -601,7 +718,7 @@ def getInitListWhoHold():
  #  name = symbol
  #else:
  #  url = "https://www.set.or.th/set/companyprofile.do?symbol="+symbol+"&ssoPageId=4&language=th&country=TH"
- #  r = requests.get(url,headers)
+ #  r = requests.post(url,headers)
  #  soup = BeautifulSoup(r.content, "lxml")
  #  name = soup.find_all("div", {"class" :"col-xs-12 col-md-12 col-lg-8"} )[0].find_all("h3")[0].text.split(":")[1].lstrip().rstrip()
 
@@ -619,7 +736,7 @@ def getInitListWhoHold():
  for s in tqdm(df.Symbol):
   s = s.strip()
   url = "https://www.set.or.th/set/companyholder.do?symbol="+s+"&ssoPageId=6&language=th&country=TH"
-  r = requests.get(url,headers)
+  r = requests.post(url,headers)
   soup = BeautifulSoup(r.content, "lxml")
   if(len(soup.find_all("div",attrs={'style':'text-align:center;color:red;'}))==0):
    listshare = soup.find_all("table",{"class":"table table-hover table-info-wrap"})
@@ -678,7 +795,7 @@ def loadDB_set_mai():
 def findCASH(s):
  symbol = s
  #load data
- r = requests.get("https://www.set.or.th/set/factsheet.do?symbol="+symbol+"&ssoPageId=3&language=th&country=TH")
+ r = requests.post("https://www.set.or.th/set/factsheet.do?symbol="+symbol+"&ssoPageId=3&language=th&country=TH")
  r.content
  soup = BeautifulSoup(r.content, "lxml")  
  if len(soup.find_all("table", {"class" : "table-factsheet-padding0"}))==0:
@@ -705,7 +822,7 @@ def findCASH(s):
 
 ############## CommonSizeBS ############################
 def commonsizeBS(symbol):
- r = requests.get("https://www.set.or.th/set/companyfinance.do?type=balance&symbol="+symbol+"&language=th&country=TH")
+ r = requests.post("https://www.set.or.th/set/companyfinance.do?type=balance&symbol="+symbol+"&language=th&country=TH")
  r.content
  soup = BeautifulSoup(r.content, "lxml") 
    
@@ -747,7 +864,7 @@ def commonsizeBS(symbol):
 
 ############## CommonSizeIncomeStatement ############################
 def commonsizeIS(symbol):
-   r = requests.get("https://www.set.or.th/set/companyfinance.do?type=income&symbol="+symbol+"&language=th&country=TH")
+   r = requests.post("https://www.set.or.th/set/companyfinance.do?type=income&symbol="+symbol+"&language=th&country=TH")
    r.content
    soup = BeautifulSoup(r.content, "lxml") 
    
@@ -1146,7 +1263,7 @@ def getFinanceRatio(liststock):
 
 def getFinanceRatioProcess(symbol): 
    headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}
-   r = requests.get("https://www.set.or.th/set/factsheet.do?symbol="+symbol+"&language=en&country=US",headers)
+   r = requests.post("https://www.set.or.th/set/factsheet.do?symbol="+symbol+"&language=en&country=US",headers)
    r.content
    soup = BeautifulSoup(r.content, "lxml") 
 
@@ -1341,7 +1458,7 @@ def getBalanceSheetCompare(symbol,period="now"):
 def getBalanceSheet(symbol):  
  symbolMofi = symbol.replace("&","%26") 
  headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}   
- r = requests.get("https://www.set.or.th/set/factsheet.do?symbol="+symbolMofi,headers)
+ r = requests.post("https://www.set.or.th/set/factsheet.do?symbol="+symbolMofi,headers)
  r.content
  soup = BeautifulSoup(r.content, "lxml") 
 
@@ -1359,7 +1476,16 @@ def getBalanceSheet(symbol):
  elif("งบแสดงฐานะการเงิน (ลบ.)" in tmpdata[18].text):
     g_data = tmpdata[18].find_all("td")
     findStep = tmpdata[18].find_all("tr")
-    
+ elif("งบแสดงฐานะการเงิน (ลบ.)" in tmpdata[17].text):
+    g_data = tmpdata[17].find_all("td")
+    findStep = tmpdata[17].find_all("tr")
+ elif("งบแสดงฐานะการเงิน (ลบ.)" in tmpdata[16].text):
+    g_data = tmpdata[16].find_all("td")
+    findStep = tmpdata[16].find_all("tr")
+ elif("งบแสดงฐานะการเงิน (ลบ.)" in tmpdata[15].text):
+    g_data = tmpdata[15].find_all("td")
+    findStep = tmpdata[15].find_all("tr")      
+
  ds  = []
  row = []
  cnt = 1
@@ -1413,7 +1539,7 @@ def getBalanceSheet(symbol):
 # ========= ev/ebitda =====================
 def evperebitda(symbol):
   if type(symbol) is str:
-    r = requests.get("https://quotes.wsj.com/TH/XBKK/"+symbol+"/financials")
+    r = requests.post("https://quotes.wsj.com/TH/XBKK/"+symbol+"/financials")
     r.content
     soup = BeautifulSoup(r.content, "lxml")  
     g_data = soup.find_all("table", {"class" : "cr_dataTable cr_sub_valuation"})
@@ -1431,7 +1557,7 @@ def evperebitda(symbol):
    df = pd.DataFrame(columns=["EV/EBITDA"])  
    for i in range(len(symbol)):
      symbol_ = symbol[i]
-     r = requests.get("https://quotes.wsj.com/TH/XBKK/"+symbol_+"/financials")
+     r = requests.post("https://quotes.wsj.com/TH/XBKK/"+symbol_+"/financials")
      r.content
      soup = BeautifulSoup(r.content, "lxml")  
      g_data = soup.find_all("table", {"class" : "cr_dataTable cr_sub_valuation"})
@@ -1449,14 +1575,24 @@ def evperebitda(symbol):
 
 
 # ==============================
+''' Error ไปแล้ว 28 may 2020 เหมือนว่า wsj เปลี่ยนระบบ 
 def getProfitAbility(symbol):   
  if type(symbol) is str:   
   symbol = symbol.lower()  
-  r = requests.get("https://quotes.wsj.com/TH/XBKK/"+symbol+"/financials")
+  #https://www.wsj.com/market-data/quotes/TH/XBKK/ADVANC/financials  
+  #r = requests.post("https://quotes.wsj.com/TH/XBKK/"+symbol+"/financials")
+  url = "https://www.wsj.com/market-data/quotes/TH/XBKK/"+symbol+"/financials"
+  
+  headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}   
+
+  r = requests.post(url,headers)
   r.content
+  
   soup = BeautifulSoup(r.content, "lxml")  
-  g_data = soup.find_all("table", {"class" : "cr_dataTable cr_sub_profitability"})
- 
+  g_data = soup.find_all("table",{"class" : "cr_dataTable cr_sub_profitability"})  
+   
+  print(soup)
+
   lb = g_data[0].find_all("span",{"class":"data_lbl"})
   data = g_data[0].find_all("span",{"class":"data_data"})   
 
@@ -1478,7 +1614,7 @@ def getProfitAbility(symbol):
   datasource = pd.DataFrame()      
   for i in range(len(symbol)):
     tmp = symbol[i].lower()    
-    r = requests.get("https://quotes.wsj.com/TH/XBKK/"+tmp+"/financials")
+    r = requests.post("https://quotes.wsj.com/TH/XBKK/"+tmp+"/financials")
     r.content
     soup = BeautifulSoup(r.content, "lxml")  
     g_data = soup.find_all("table", {"class" : "cr_dataTable cr_sub_profitability"})
@@ -1505,6 +1641,7 @@ def getProfitAbility(symbol):
     c.append(symbol[i]+".rate")
   datasource.columns = c  
  return datasource
+'''
 
 # ==============================
 def getohlc(symbol):
@@ -1537,7 +1674,7 @@ def getMemberOfIndex(indexType):
 
  headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}   
                   
- r = requests.get("https://marketdata.set.or.th/mkt/sectorquotation.do?sector="+symbol+"&language=th&country=TH",verify=False,headers=headers)
+ r = requests.post("https://marketdata.set.or.th/mkt/sectorquotation.do?sector="+symbol+"&language=th&country=TH",verify=False,headers=headers)
  r.content
  soup = BeautifulSoup(r.content, "lxml")  
  dtmpCheck = soup.find_all("table", {"class" : "table-info"})
@@ -1576,7 +1713,7 @@ class portfolio:
     
 def getLastPrice(symbol,date="now",ordersymbol=False):
  if type(symbol) is str: 
-   r = requests.get("https://marketdata.set.or.th/mkt/stockquotation.do?symbol="+symbol+"&ssoPageId=1&language=th&country=TH")
+   r = requests.post("https://marketdata.set.or.th/mkt/stockquotation.do?symbol="+symbol+"&ssoPageId=1&language=th&country=TH")
    r.content
    soup = BeautifulSoup(r.content, "lxml")  
    g_data = soup.find_all("table", {"class" : "table table-hover table-set col-3-center table-set-border-yellow"})
@@ -1588,7 +1725,7 @@ def getLastPrice(symbol,date="now",ordersymbol=False):
  elif type(symbol) is list:
    ds = []     
    for i in symbol:
-     r = requests.get("https://marketdata.set.or.th/mkt/stockquotation.do?symbol="+i+"&ssoPageId=1&language=th&country=TH")
+     r = requests.post("https://marketdata.set.or.th/mkt/stockquotation.do?symbol="+i+"&ssoPageId=1&language=th&country=TH")
      r.content
      soup = BeautifulSoup(r.content, "lxml")  
      g_data = soup.find_all("table", {"class" : "table table-hover table-set col-3-center table-set-border-yellow"})
@@ -1620,7 +1757,7 @@ def trimdata(s):
 
 def listStockInSector(sample_in_sector):
  headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}   
- r = requests.get("http://www.settrade.com/C04_08_stock_sectorcomparison_p1.jsp?txtSymbol="+sample_in_sector,headers=headers)
+ r = requests.post("http://www.settrade.com/C04_08_stock_sectorcomparison_p1.jsp?txtSymbol="+sample_in_sector,headers=headers)
  r.content
  soup = BeautifulSoup(r.content, "lxml")  
  g_data = soup.find_all("table", {"class" : "table table-info"})
@@ -1646,7 +1783,7 @@ def getdata(symbol,returntype="ar"):
  symboltmp = symbol.replace('&', '%26') 
  datasource = []   
  headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}      
- r = requests.get("http://www.settrade.com/C04_06_stock_financial_p1.jsp?txtSymbol="+symboltmp,headers)
+ r = requests.post("http://www.settrade.com/C04_06_stock_financial_p1.jsp?txtSymbol="+symboltmp,headers)
  r.content
  soup = BeautifulSoup(r.content, "lxml")  
  g_data = soup.find_all("table", {"class" : "table table-hover table-info"})[0].find_all("tr")
@@ -1657,14 +1794,23 @@ def getdata(symbol,returntype="ar"):
     td = k.find_all("td")
     for m in td:
        tmp = m.text.strip()
+  
        if(tmp!=""):
         tmp = tmp.replace(',', '') 
-          
+        
         if(tmp=="สินทรัพย์รวม"):
            col.append(tmp)
            lenindex = len(td)
            tmp = k.find_all("td")[lenindex-1].text.strip()
            tmp = trimdata(tmp)
+         
+           if(tmp==""):
+             tmp = k.find_all("td")[lenindex-2].text.strip()
+             tmp = trimdata(tmp)
+           
+           if(tmp=="-"): #fix bug 0.15541 กรณี ข้อมูลส่งมาเป็น "-"
+             tmp = np.nan
+
            if(tmp=="" or lenindex<=2): #เป็นพวกกองทุนรวม
              return 0
            datasource.append(float(tmp))   
@@ -1675,6 +1821,14 @@ def getdata(symbol,returntype="ar"):
            lenindex = len(td)
            tmp = k.find_all("td")[lenindex-1].text.strip()
            tmp = trimdata(tmp)
+           
+           if(tmp==""):
+             tmp = k.find_all("td")[lenindex-2].text.strip()
+             tmp = trimdata(tmp)
+           
+           if(tmp=="-"): #fix bug 0.15541 กรณี ข้อมูลส่งมาเป็น "-"
+             tmp = np.nan
+
            if(tmp=="" or lenindex<=2): #เป็นพวกกองทุนรวม
              return 0
            datasource.append(float(tmp))   
@@ -1685,6 +1839,14 @@ def getdata(symbol,returntype="ar"):
            lenindex = len(td)
            tmp = k.find_all("td")[lenindex-1].text.strip()
            tmp = trimdata(tmp)
+            
+           if(tmp==""):
+             tmp = k.find_all("td")[lenindex-2].text.strip()
+             tmp = trimdata(tmp)
+           
+           if(tmp=="-"): #fix bug 0.15541 กรณี ข้อมูลส่งมาเป็น "-"
+             tmp = np.nan
+
            if(tmp=="" or lenindex<=2): #เป็นพวกกองทุนรวม
              return 0
            datasource.append(float(tmp))   
@@ -1695,8 +1857,17 @@ def getdata(symbol,returntype="ar"):
            lenindex = len(td)
            tmp = k.find_all("td")[lenindex-1].text.strip()
            tmp = trimdata(tmp)
+            
+           if(tmp==""):
+             tmp = k.find_all("td")[lenindex-2].text.strip()
+             tmp = trimdata(tmp)
+           
+           if(tmp=="-"): #fix bug 0.15541 กรณี ข้อมูลส่งมาเป็น "-"
+             tmp = np.nan
+
            if(tmp==""): #เป็นพวกกองทุนรวม
              return 0
+            
            datasource.append(float(tmp))   
            break
 
@@ -1705,6 +1876,15 @@ def getdata(symbol,returntype="ar"):
            lenindex = len(td)
            tmp = k.find_all("td")[lenindex-1].text.strip()
            tmp = trimdata(tmp)
+           
+          
+           if(tmp==""):
+             tmp = k.find_all("td")[lenindex-2].text.strip()
+             tmp = trimdata(tmp)
+
+           if(tmp=="-"): #fix bug 0.15541 กรณี ข้อมูลส่งมาเป็น "-"
+             tmp = np.nan
+              
            if(tmp==""): #เป็นพวกกองทุนรวม
              return 0
            datasource.append(float(tmp))   
@@ -1715,14 +1895,18 @@ def getdata(symbol,returntype="ar"):
            lenindex = len(td)
            tmp = k.find_all("td")[lenindex-1].text.strip()
            tmp = trimdata(tmp)
-           if(tmp==""): #เป็นพวกกองทุนรวม
-             return 0
             
+           if(tmp==""):
+             tmp = k.find_all("td")[lenindex-2].text.strip()
+             tmp = trimdata(tmp)
+                   
            if(tmp=='N/A' or tmp=='N.A.'):
               tmp = np.nan  
            if(tmp=="-"): #fix bug 0.15541 กรณี ข้อมูลส่งมาเป็น "-"
              tmp = np.nan
 
+           if(tmp==""): #เป็นพวกกองทุนรวม
+             return 0     
            datasource.append(float(tmp))   
            break
             
@@ -1731,6 +1915,11 @@ def getdata(symbol,returntype="ar"):
            lenindex = len(k.find_all("td"))
            tmp = k.find_all("td")[lenindex-1].text.strip()
            tmp = trimdata(tmp) 
+            
+           if(tmp==""):
+             tmp = k.find_all("td")[lenindex-2].text.strip()
+             tmp = trimdata(tmp)
+          
            if(tmp=="-"): #fix bug 0.15541 กรณี ข้อมูลส่งมาเป็น "-"
              tmp = np.nan
            datasource.append(float(tmp))   
@@ -1741,6 +1930,11 @@ def getdata(symbol,returntype="ar"):
            lenindex = len(k.find_all("td"))
            tmp = k.find_all("td")[lenindex-1].text.strip()
            tmp = trimdata(tmp) 
+            
+           if(tmp==""):
+             tmp = k.find_all("td")[lenindex-2].text.strip()
+             tmp = trimdata(tmp)
+            
            if(tmp=='N/A' or tmp=='N.A.'):
               tmp = np.nan 
            if(tmp=="-"): #fix bug 0.15541 กรณี ข้อมูลส่งมาเป็น "-"
@@ -1754,6 +1948,11 @@ def getdata(symbol,returntype="ar"):
            lenindex = len(k.find_all("td"))
            tmp = k.find_all("td")[lenindex-1].text.strip()
            tmp = trimdata(tmp) 
+            
+           if(tmp==""):
+             tmp = k.find_all("td")[lenindex-2].text.strip()
+             tmp = trimdata(tmp)
+            
            if(tmp=='N/A'):
               tmp = 0  
            if(tmp=="-"): #fix bug 0.15541 กรณี ข้อมูลส่งมาเป็น "-"
@@ -1766,18 +1965,24 @@ def getdata(symbol,returntype="ar"):
            lenindex = len(k.find_all("td"))
            tmp = k.find_all("td")[lenindex-1].text.strip()
            tmp = trimdata(tmp) 
+            
+           if(tmp==""):
+             tmp = k.find_all("td")[lenindex-2].text.strip()
+             tmp = trimdata(tmp)
+            
            if(tmp=='N/A' or tmp=='N.A.'):
               tmp = np.nan  
            if(tmp=="-"): #fix bug 0.15541 กรณี ข้อมูลส่งมาเป็น "-"
              tmp = np.nan    
            datasource.append(float(tmp))   
            break
-          
+ 
+ 
  if returntype=="df":
      datasource = pd.DataFrame(datasource).T
      datasource.columns = ["symbol","สินทรัพย์รวม","หนี้สินรวม","ส่วนของผู้ถือหุ้น","รายได้รวม","กำไรสุทธิ","ROE(%)","มูลค่าหลักทรัพย์ตามราคาตลาด","P/E (เท่า)","P/BV (เท่า)","อัตราส่วนเงินปันผลตอบแทน(%)"]  
             
- return datasource      
+ return datasource 
 
 
 def ar2df(ar):
@@ -1835,7 +2040,7 @@ def fillHistData(symbol,source):
   url = "https://www.set.or.th/set/historicaltrading.do?symbol="+symbol
 
   headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}
-  r = requests.get(url,headers)
+  r = requests.post(url,headers)
   soup = BeautifulSoup(r.content, "lxml") 
 
   ####
